import CharmCord.CharmErrorHandling as ErrorHandling

EH = ErrorHandling.CharmErrorHandling()


async def userMention(user, context):
    from CharmCord.Classes.CharmCord import bots

    try:
        int(user)
        new_user = await bots.fetch_user(user)
    except ValueError:
        EH.Errors(1, user)
        return
    return new_user.mention



import os


def load_commands(dir):
    for filename in os.listdir(dir):
        if filename.endswith(".py"):
            exec(f"from {dir.replace('/', '.')}.{filename[:-3]} import *")
        elif os.path.isdir(f"{dir}/{filename}") and not filename.endswith("__"):
            load_commands(f"{dir}/{filename}")



import CharmCord.CharmErrorHandling as ErrorHandling
import discord
EH = ErrorHandling.CharmErrorHandling()


async def guildTextChannels(ID, Context):
    from CharmCord.Classes.CharmCord import bots

    try:
        ID = int(ID)
        text = []
        guild = await bots.fetch_guild(ID)
        channels = list(await guild.fetch_channels())
        for channel in channels:
            if isinstance(channel, discord.TextChannel):
                text.append(channel.name)
        return text
    except ValueError:
        EH.Errors(2, "None")



import asyncio
import random
from enum import Enum

# class CharmActivity(Enum):
import discord
from discord.ext import tasks


def setActivity(message: str, type: str = "watching"):
    if type.lower() == "watching":
        act = discord.Activity(type=discord.ActivityType.watching, name=message)
    elif type.lower() == "playing":
        act = discord.Activity(type=discord.ActivityType.playing, name=message)
    else:
        act = discord.Activity(type=discord.ActivityType.listening, name=message)

    return act


def loopActivity(time: int, messages: [str], types: str = "watching"):
    @tasks.loop(seconds=time)
    async def updateActivity(message=messages, type=types):
        from CharmCord.Classes.CharmCord import bots
        await bots.change_presence(
            activity=discord.Activity(
                type=discord.ActivityType.watching, name=random.choice(message)
            )
        )

# Still having a lot of errors with this, will return when I have the time to set it up



import json


async def setUserVar(args, Context):
    ag = args.split(";")
    user = ag[0]
    var = ag[1]
    value = ag[2]
    with open("variables.json", "r") as vars:
        total = json.load(vars)
    with open("variables.json", "w") as vars:
        try:
            total.update({f"{user}_{var}": int(value)})
        except:
            total.update({f"{user}_{var}": value})
        json.dump(total, vars)
    return



import CharmCord.CharmErrorHandling as ErrorHandling

EH = ErrorHandling.CharmErrorHandling()


async def channelChangedRoles(ID, Context):
    """
    Ex. $channelChangedRoles[ChannelID]
    """
    if len(ID) < 1:
        raise EH.Errors(4, "No parameter provided for '$channelChangedRoles'")
    from CharmCord.Classes.CharmCord import bots

    if ";" in ID:
        args = ID.split(";")
        ID = args[0]
        Index = args[1]
        try:
            int(ID)
            channel = await bots.fetch_channel(ID)
            return channel.changed_roles[int(Index) - 1]
        except ValueError:
            EH.Errors(2, ID)
    else:
        try:
            int(ID)
            channel = await bots.fetch_channel(ID)
            return channel.changed_roles
        except ValueError:
            EH.Errors(2, ID)



from CharmCord.tools import checkArgCheck, checkArgs, findBracketPairs, noArguments, isValid

AC = {}


class Charmcogs:
    def commandcogs(self, name, code):
        from .CharmCord import bots

        @bots.command(name=name)
        async def go(ctx, *args, code=code):
            from CharmCord.Classes.CharmCord import TotalFuncs

            Context = ctx
            newcode = await checkArgCheck(args, code, Context)
            if newcode == "Failed":
                return
            finalcode = await noArguments(newcode, TotalFuncs, Context)
            finalcode = checkArgs(args, finalcode)
            finalcode = await isValid(finalcode, TotalFuncs)
            await findBracketPairs(finalcode, TotalFuncs, Context)

    def slashcommandcogs(self, name, code, args: list, description):
        def slashcommand():
            from CharmCord.Classes.CharmCord import bots
            newArgs = []
            for i in args:
                newArgs.append(f"{i}: str")
            needs = {"arguments": args, "codes": code, "bots": bots, "name": name, "description": description}
            func = f"""@bots.tree.command(name=name, description=description)
async def go(ctx, {', '.join(newArgs)}):
                        from CharmCord.Classes.CharmCord import TotalFuncs
                        from CharmCord.tools import noArguments, slashArgs, findBracketPairs
                        Context = ctx
                        new = []
                        for i in arguments:
                            new.append(eval(i))
                        finalCode = await noArguments(codes, TotalFuncs, Context)
                        finalCode = slashArgs(new, finalCode)
                        await findBracketPairs(finalCode, TotalFuncs, Context)
                """
            exec(func, needs)
        slashcommand()


import CharmCord.CharmErrorHandling as ErrorHandling

EH = ErrorHandling.CharmErrorHandling()


async def channelCreated(args: str, Context, timezones, format_datetime):
    """
    Ex. $channelCreated[ChannelID]
    """
    if len(args) < 1:
        raise EH.Errors(4, "No parameter provided for '$channelCreated'")
    est, utc, pst = timezones
    try:
        ID, TIME, FORM = tuple(args.split(";"))
        TIME = locals()[TIME.strip()]
        FORM = FORM.lower()
    except ValueError:
        FORM = "full"
        try:
            ID, TIME = tuple(args.split(","))
            TIME = locals()[TIME.strip()]
        except:
            ID = args
            TIME = utc

    from CharmCord.Classes.CharmCord import bots

    try:
        int(ID)
        channel = await bots.fetch_channel(ID)

        desiredDateForm = format_datetime(channel.created_at, FORM, TIME)
        if desiredDateForm != "ERROR":
            return desiredDateForm
        else:
            raise SyntaxError("Invalid Format option in $channelCreated!")

    except ValueError:
        EH.Errors(2, ID)



async def messageContent(IDs, Context):
    from CharmCord.Classes.CharmCord import bots

    try:
        args = IDs.split(";")
        channel = await bots.fetch_channel(args[0])
        mes = int(args[1])
        message = await channel.fetch_message(mes)
        return message.content
    except:
        raise SyntaxError("Not a message ID")



import discord


class SlashCommands:

    def slashCommand(self, name, code, args: list, description=None):
        from CharmCord.Classes.CharmCord import bots
        newArgs = []
        for i in args:
            newArgs.append(f"{i}: str")
        needs = {"arguments": args, "codes": code, "bots": bots, "name": name, "description": description}
        func = f"""
@bots.tree.command(name=name, description=description)
async def go(ctx, {', '.join(newArgs)}):
                from CharmCord.Classes.CharmCord import TotalFuncs
                from CharmCord.tools import noArguments, slashArgs, findBracketPairs
                Context = ctx
                new = []
                for i in arguments:
                    new.append(eval(i))
                finalCode = await noArguments(codes, TotalFuncs, Context)
                finalCode = slashArgs(new, finalCode)
                await findBracketPairs(finalCode, TotalFuncs, Context)
        """
        exec(func, needs)



import asyncio

import discord
from discord.ext import commands
from CharmCord.tools import FunctionHandler
from .CommandHandler import load_commands

global TotalFuncs
global bots
global all_vars


class CharmCord:
    # Global variables
    global bots
    global all_vars

    def __init__(
            self,
            prefix,
            case_insensitive,
            intents: tuple,
            activity,
            help_command,
            load_command_dir,
    ):
        # Global variables
        global bots

        # Initialize Start class
        self.prefix = prefix
        self.case_insensitive = case_insensitive
        self.intented = intents
        self._help_command = help_command
        self._clients = ""
        self.intent = ""
        self._activity = activity
        self.all_variables = {}

        # Determine intents
        if "all" in self.intented:
            self.intent = discord.Intents.all()
        elif "default" in self.intented:
            self.intent = discord.Intents.default()
        else:
            self.intent = discord.Intents.default()

        # Enable specific intents
        if "message" in self.intented:
            self.intent.message_content = True
        if "members" in self.intented:
            self.intent.members = True
        if "presences" in self.intented:
            self.intent.presences = True

        # Create bot instances
        if self._activity is None:
            self._clients = commands.Bot(
                command_prefix=self.prefix,
                case_insensitive=self.case_insensitive,
                intents=self.intent,
                help_command=self._help_command,
            )
            bots = self._clients

        else:
            self._clients = commands.Bot(
                command_prefix=self.prefix,
                case_insensitive=self.case_insensitive,
                intents=self.intent,
                activity=self._activity,
                help_command=self._help_command,
            )
            bots = self._clients

        try:
            load_commands(load_command_dir)
        except FileNotFoundError:
            pass

        import json

        try:
            with open("variables.json", "r") as var:
                pass
        except FileNotFoundError:
            with open("variables.json", "w") as var:
                go = {"STRD": True}
                json.dump(go, var)

    def run(self, token: str):
        bots.run(token)

    def variables(self, vars: dict):
        global all_vars
        for key, value in vars.items():
            self.all_variables[key] = value
        all_vars = self.all_variables

    def slashCommand(self, name: str, code: str, args: list = [], description: str = ""):
        from .SlashCommands import SlashCommands

        sl = SlashCommands().slashCommand
        sl(name=name, code=code, args=args, description=description.lower())

    def command(self, name: str, code: str, aliases=[str]):
        from .Commands import Commands

        co = Commands().command
        co(Name=name, Code=code, Aliases=aliases)

    def onChannelUpdated(self, Code):
        @bots.event
        async def on_guild_channel_update(before, after):
            from CharmCord.Functions.Events.oldChannel import options as old

            for i in old.keys():
                old[i] = before.i

            if Code is not None:
                from CharmCord.tools import findBracketPairs

                await findBracketPairs(Code, TotalFuncs, None)

    def onChannelDeleted(self, Code=None):
        @bots.event
        async def on_guild_channel_delete(channel):
            from CharmCord.Functions.Events.deletedChannel import options

            options["name"] = channel.name
            options["id"] = channel.id
            # more options coming
            if Code is not None:
                from CharmCord.tools import findBracketPairs

                await findBracketPairs(Code, TotalFuncs, None)

    def onReady(self, Code):
        @bots.event
        async def on_ready():
            from CharmCord.tools import findBracketPairs, noArguments
            from CharmCord.CharmErrorHandling import CharmCord_Errors
            finalCode = await noArguments(Code, TotalFuncs, None)
            await findBracketPairs(finalCode, TotalFuncs, None)
            try:
                await bots.tree.sync()
            except:
                CharmCord_Errors("All slash commands need a description")


def CharmClient(
        prefix: str,
        case_insensitive: bool = False,
        intents: tuple = ("default",),
        activity=None,
        help_command=None,
        load_command_dir="commands",
):
    """
    CharmCord Discord Client
    """
    # Global variables
    global bots
    global TotalFuncs

    # Initialize FunctionHandler and register functions
    Functions = FunctionHandler()
    TotalFuncs = Functions
    Functions.register_functions()

    # Create Start instance and return working bot
    _final = CharmCord(
        prefix, case_insensitive, intents, activity, help_command, load_command_dir
    )
    working = _final
    return working



import CharmCord.CharmErrorHandling as ErrorHandling

EH = ErrorHandling.CharmErrorHandling()


async def channelCategoryID(empty, Context):
    """
    Ex. $channelCategoryID
    Returns the ID of the current category ID
    """
    pass

    try:
        int(Context.channel.category.id)
    except ValueError:
        EH.Errors(3, "None")
    return Context.channel.category.id



import json


async def getUserVar(args, Context):
    from CharmCord.Classes.CharmCord import all_vars

    ag = args.split(";")
    user = ag[0]
    var = ag[1]
    try:
        with open("variables.json", "r") as vars:
            total = json.load(vars)
            return total[f"{user}_{var}"]
    except KeyError:
        with open("variables.json", "r") as vars:
            total = json.load(vars)
        with open("variables.json", "w") as vars:
            total.update({f"{user}_{var}": all_vars[var]})
            json.dump(total, vars)
            return all_vars[var]



import CharmCord.CharmErrorHandling as ErrorHandling

EH = ErrorHandling.CharmErrorHandling()


async def guildName(ID, Context):
    if len(ID) < 1:
        raise EH.Errors(4, "No parameter provided for '$guildName'")
    from CharmCord.Classes.CharmCord import bots

    try:
        int(ID)
        guild = await bots.fetch_guild(ID)
        return guild.name
    except ValueError:
        EH.Errors(2, ID)



class CharmErrorHandling:
    def Errors(self, error, code):
        """
        ERRORS:

        1: user     ID
        2: channel  ID
        3: category ID

        4: WILDCARD
        """

        if error == 1:
            raise SyntaxError(f"ID '{code}' is not a user ID")
        if error == 2:
            raise SyntaxError(f"ID '{code}' is not a channel ID")
        if error == 3:
            raise SyntaxError(f"ID '{code}' is not a category ID")

        if error == 4:
            raise SyntaxError(code)


class CharmCord_Errors(Exception):
    def __init__(self, Error: str):
        raise SyntaxError(Error)


import CharmCord.CharmErrorHandling as ErrorHandling
from CharmCord.CharmErrorHandling import CharmCord_Errors

EH = ErrorHandling.CharmErrorHandling()


async def channelMention(ID, Context):
    """
    Ex. $channelMention[ChannelID]
    returns a channel mention from the given ID
    """
    if len(ID) < 1:
        raise EH.Errors(4, "No parameter provided for '$channelMention'")
    from CharmCord.Classes.CharmCord import bots

    try:
        int(ID)
        channel = await bots.fetch_channel(ID)
        return channel.mention
    except ValueError:
        CharmCord_Errors(f"$channelMention: {ID} not valid channel id\nCommand: {Context.command.name}")



from CharmCord.tools import checkArgCheck, checkArgs, findBracketPairs, noArguments, lets, isValid

########################################
#              COMMANDS                #
########################################


class Commands:
    # Global variables

    def command(self, Name, Code, Aliases=[]):
        # Define command function dynamically
        from CharmCord.Classes.CharmCord import bots

        @bots.command(name=Name, aliases=Aliases)
        async def go(ctx, *args, Code=Code):
            from CharmCord.Classes.CharmCord import TotalFuncs

            Context = ctx
            newCode = await checkArgCheck(args, Code, Context)
            if newCode == "Failed":
                return
            finalCode = await noArguments(newCode, TotalFuncs, Context)
            finalCode = checkArgs(args, finalCode)
            finalCode = await isValid(finalCode, TotalFuncs)
            await findBracketPairs(finalCode, TotalFuncs, Context)
            if len(lets) >= 1:
                lets.clear()




import setuptools

with open("README.md", "r") as rd:
    longtext = rd.read()

setuptools.setup(
    # Here is the module name.
    name="CharmCord",
    version="0.18.5",
    author="Jade",
    description="CharmCord is the best python string-based package for Discord bot devs",
    long_description=longtext,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    # if module has dependencies i.e. if your package rely on other package at pypi.org
    # then you must add there, in order to download every requirement of package
    install_requires=["discord.py", "pytz"],
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import CharmCord.CharmErrorHandling as ErrorHandling

EH = ErrorHandling.CharmErrorHandling()


async def channelCategoryName(ID, Context):
    """
    Ex. $channelCategoryName[ChannelID]
    returns the name of the category name of the given ID
    """
    if len(ID) < 1:
        raise EH.Errors(4, "No parameter provided for '$channelCategoryName'")
    from CharmCord.Classes.CharmCord import bots

    try:
        int(ID)
        channel = await bots.fetch_channel(ID)
        return channel.category.name
    except ValueError:
        EH.Errors(3, ID)



#######################################
#              EVENTS                 #
#######################################

#
# class AoiEvents:
#    def onReady(self, code):
#        from Aoipy.Classes.AoiPyClient import bots, TotalFuncs
#        # Define on_ready event function
#        @bots.event
#        async def on_ready():
#            await findBracketPairs(code, TotalFuncs, None)
#
#    def onGuildChannelDelete(self):
#        from Aoipy.Classes.AoiPyClient import bots
#        from Aoipy.Functions.Events.onChannelDelete import options
#        @bots.event
#        async def on_guild_channel_delete(channel):
#            options["name"] = channel.name
#            options['id'] = channel.id
#
#
#



import CharmCord.CharmErrorHandling as ErrorHandling

EH = ErrorHandling.CharmErrorHandling()


async def channelName(ID, Context):
    """
    Ex. $channelName[ChannelID]
    returns channel name of the given ID
    """
    if len(ID) < 1:
        raise EH.Errors(4, "No parameter provided for '$channelName'")
    from CharmCord.Classes.CharmCord import bots

    try:
        int(ID)
        channel = await bots.fetch_channel(ID)
        return channel.name
    except ValueError:
        EH.Errors(2, ID)



from CharmCord.all_functions import newline_char


async def slashSend(args: str, Context):
    try:
        message = args
        message = message.replace(newline_char, "\n")
        await Context.response.send_message(message)
    except:
        raise SyntaxError("Can't send empty message!")
    return
